import java.io.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
//import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class MuliniTransformer {

	public Document getXMLDoc(String targetName, String lang, int step)
		throws FileNotFoundException, IOException, ParserConfigurationException, SAXException {
		
		Integer istep = new Integer(step);

		InputSource inXML = null;

		inXML = new InputSource(new FileInputStream("../temp/" + lang + "/" + targetName + istep.toString() + ".xml"));

		DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
		dfactory.setNamespaceAware(true);
		Document doc = dfactory.newDocumentBuilder().parse(inXML);

		return doc;
	}

	public void setXML(Node srcNode, String targetName, String lang, int step)
		throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException {

		Integer istep = new Integer(step);
		
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");

		transformer.transform(new DOMSource(srcNode), 
			new StreamResult(new FileOutputStream("../temp/" + lang + "/" + targetName + istep.toString() + ".xml")));	
	}
	
	public int buildToXML(String targetName, String template, String lang, int step, String aspect)
		throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException {

		Integer istep = new Integer(step);
		int nextStep = step + 1;
		Integer iNextStep = new Integer(nextStep);
		
		TransformerFactory tFactory = TransformerFactory.newInstance();

		Transformer transformer = tFactory.newTransformer(new StreamSource("../templates/" + lang + "/" + template));

		if (aspect != null)
			transformer.setParameter("aspect_name", aspect);

		transformer.transform(new StreamSource("../temp/" + lang + "/" + targetName + istep.toString() + ".xml"), 
			new StreamResult(new FileOutputStream("../temp/" + lang + "/" + targetName + iNextStep.toString() + ".xml")));

		return nextStep;
	}

	public int buildToXML(String targetName, String template, String lang, int step, String docName, String docLoc)
		throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException {

		Integer istep = new Integer(step);
		int nextStep = step + 1;
		Integer iNextStep = new Integer(nextStep);
		
		TransformerFactory tFactory = TransformerFactory.newInstance();

		Transformer transformer = tFactory.newTransformer(new StreamSource("../templates/" + lang + "/" + template));

		transformer.setParameter("doc_name", docName);
		transformer.setParameter("doc_loc", docLoc);

		transformer.transform(new StreamSource("../temp/" + lang + "/" + targetName + istep.toString() + ".xml"), 
			new StreamResult(new FileOutputStream("../temp/" + lang + "/" + targetName + iNextStep.toString() + ".xml")));

		return nextStep;
	}

	public void buildToSource(String className, String lang, int depth) 
		throws TransformerException, FileNotFoundException, IOException, ParserConfigurationException, SAXException {
		
		Document finalDoc = getXMLDoc(className, lang, depth);

		NodeIterator files = XPathAPI.selectNodeIterator(finalDoc, "//file");
		Node file = null; 
		while ((file = files.nextNode())!= null) {
			Node loc = XPathAPI.selectSingleNode(file, "./name/@loc");
			Node name = XPathAPI.selectSingleNode(file, "./name/@id");
			Node content = XPathAPI.selectSingleNode(file, "./content");

			File filePath = new File("../outputs/" + loc.getNodeValue());
			filePath.mkdir();

			FileWriter fw = new FileWriter("../outputs/" + loc.getNodeValue() + "/" + name.getNodeValue());			

			generateSource(content, fw);

			fw.close();
		}	
	}

	private void generateSource(Node c, FileWriter fw) throws IOException {
		NodeList nl = c.getChildNodes();
		Node cn = null;

		for (int i = 0; i < nl.getLength(); i++) {
			cn = nl.item(i);
			if (isTextNode(cn)) {
				String line = null;

				BufferedReader br = new BufferedReader(new StringReader(cn.getNodeValue()));	

				while((line = br.readLine()) != null) {
					char inputChars[] = new char[line.length() + 1];
					line.getChars(0, line.length(), inputChars, 0);
					inputChars[line.length()] = '\n';
           
					fw.write(inputChars);
				}				
				fw.flush();
			} else {
				generateSource(cn, fw);
			}
		}
	}

	static boolean isTextNode(Node n) {
		if (n == null)
			return false;
		short nodeType = n.getNodeType();
		return nodeType == Node.CDATA_SECTION_NODE || nodeType == Node.TEXT_NODE;
	}
}
